Attribute VB_Name = "GPIBCore"
' ****************************************************************************
' GPIBCore  (C) 1992-2000 Vincent Himpe All rights reserved
' Ver 5.00   18/01/2000 GPIB transfer using National Instruments GPIB-32.dll
'
' Patch history
' Patch 01 : 20-02-2000 Fixed wrong VB syntax displey for GPIBread
' patch 02 : 21-02-2000 modified GPIbtimeout for more values
' patch 03 : 24-02-2000 modified GPIBtimeout for board and device level timeout
' patch 04 : 24-02-2000 enable automatic serial poll during GPIBinit . kill all SRQ
' patch 05 : 07-03-2000 added entry to vi5_vision-instrument grid
' patch 06 : 07-03-2000 added GPIBsnoop to GPIBdebug command
' patch 07 : 09-03-2000 EOI on last byte is default for all machines now
' patch 9 : fixup for older instruments EOI on transmission on
' ****************************************************************************

Option Explicit

  Global Const gpibversion$ = "Ver 5.0"
  Global Const gpibbuild$ = "2000307-1 PATCHlevel 6"
' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
' Global variables
' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Global ibret As Double
  Global ib$
  Global vi5_DDE_Server_Running As Boolean ' notify flag for the debugging server

' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
' Timeout values and meanings
' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Global Const T1s = 11       ' Timeout of 1 s (ideal)
  Global Const T3s = 12       ' Timeout of 3 s (ideal)
  Global Const T10s = 13      ' Timeout of 10 s (ideal)
  Global Const T30s = 14      ' Timeout of 30 s (ideal)
  Global Const T100s = 15
  Global Const T300s = 16
  Global Const T1000s = 17

' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
' Internal variables for NI driver interaction
' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Dim ibsta As Integer
  Dim iberr As Integer
  Dim ibcnt As Integer
  Dim ibcntl As Long
  Dim Longibsta As Long
  Dim Longiberr As Long
  Dim Longibcnt As Long
  Dim GPIBglobalsRegistered As Integer

' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
' Internal constants for NI driver interaction
' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Const EERR = &H8000      ' Error detected
  Const EDVR = 0      ' System error
  Const EABO = 6      ' I/O operation aborted
  Const ENEB = 7      ' Non-existent interface board
  Const EFSO = 12     ' File system operation error

' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
' GPIBcore VI5 internal variables
' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Type vi5_vision_instrument
       handle As Integer    ' NI handle storage
       descriptor As String ' instrument name (given by user or detected)
       truename As String   ' real instrument name (detected)
       runstate As Boolean  ' true = in use . false : vacant
       tracing As Boolean   ' true : log commands issued to device
       deferred As Boolean  ' true : emulate machine
  End Type

  Dim vi5_instrument(32) As vi5_vision_instrument
  Dim vi5_inputbuffer As String * 65500
  Dim vi5_tmpbuffer As String
  Dim vi5_runlevel As Integer ' determines operational state of the stack
  Dim vi5_singlestep As Boolean ' allows to singlestep trough gpib commands
  Dim vi5_defer As Boolean
  Dim logdisable As Integer
  
  ' GPIBspy interface
  
  Dim vi5_spyback As String ' tells you if the request has been issued by the spy channel
  Dim vi5_spyrequest As Boolean ' temp string t store feedback from the spy channel.

' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
' WINio interface functions . Win95io (c) 1995 Vincent Himpe
' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
   Declare Function inp& Lib "win95io.dll" Alias "inpb" (ByVal x1&)
   Declare Function inpw& Lib "win95io.dll" Alias "inpdb" (ByVal x1&)
   Declare Sub out Lib "win95io.dll" (ByVal addr As Integer, ByVal dta As Integer)
   Declare Sub outw Lib "win95io.dll" (ByVal x1&, ByVal x2&)
   Declare Sub mybeep Lib "win95io.dll" Alias "beep" ()
   Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
'   NI-488.2 DLL entry function declarations
' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
   Declare Function ibconfig32 Lib "Gpib-32.dll" Alias "ibconfig" (ByVal ud As Long, ByVal opt As Long, ByVal v As Long) As Long
   Declare Function ibclr32 Lib "Gpib-32.dll" Alias "ibclr" (ByVal ud As Long) As Long
   Declare Function ibdev32 Lib "Gpib-32.dll" Alias "ibdev" (ByVal bdid As Long, ByVal pad As Long, ByVal sad As Long, ByVal tmo As Long, ByVal eot As Long, ByVal eos As Long) As Long
   Declare Function ibeos32 Lib "Gpib-32.dll" Alias "ibeos" (ByVal ud As Long, ByVal v As Long) As Long
   Declare Function ibeot32 Lib "Gpib-32.dll" Alias "ibeot" (ByVal ud As Long, ByVal v As Long) As Long
   Declare Function ibln32 Lib "Gpib-32.dll" Alias "ibln" (ByVal ud As Long, ByVal pad As Long, ByVal sad As Long, ln As Long) As Long
   Declare Function ibloc32 Lib "Gpib-32.dll" Alias "ibloc" (ByVal ud As Long) As Long
   Declare Function ibonl32 Lib "Gpib-32.dll" Alias "ibonl" (ByVal ud As Long, ByVal v As Long) As Long
   Declare Function ibrd32 Lib "Gpib-32.dll" Alias "ibrd" (ByVal ud As Long, sStr As Any, ByVal cnt As Long) As Long
   Declare Function ibrdf32 Lib "Gpib-32.dll" Alias "ibrdfA" (ByVal ud As Long, sStr As Any) As Long
   Declare Function ibsic32 Lib "Gpib-32.dll" Alias "ibsic" (ByVal ud As Long) As Long
   Declare Function ibsre32 Lib "Gpib-32.dll" Alias "ibsre" (ByVal ud As Long, ByVal v As Long) As Long
   Declare Function ibtmo32 Lib "Gpib-32.dll" Alias "ibtmo" (ByVal ud As Long, ByVal v As Long) As Long
   Declare Function ibtrg32 Lib "Gpib-32.dll" Alias "ibtrg" (ByVal ud As Long) As Long
   Declare Function ibwrt32 Lib "Gpib-32.dll" Alias "ibwrt" (ByVal ud As Long, sStr As Any, ByVal cnt As Long) As Long
   Declare Function ibwrtf32 Lib "Gpib-32.dll" Alias "ibwrtfA" (ByVal ud As Long, sStr As Any) As Long
   Declare Function RegisterGpibGlobalsForThread Lib "Gpib-32.dll" (Longibsta As Long, Longiberr As Long, Longibcnt As Long, ibcntl As Long) As Long
   Declare Function UnregisterGpibGlobalsForThread Lib "Gpib-32.dll" () As Long

' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
' VISION 5 base routines
' -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

' ------------------------------------------------------------------------------------
' VI5_DEBUG : send debug information to debugging console
'
' depending on the state of the VI5_debugger flag the output is redirected to either
' the VISion console ( VI5_DEBUGGER =1 ) or sent to the standard visual basic debug
' channel ( immediate window (CTRL-G) )
' To enable the external debugging you must also include the GPIBserver.FRM in your
' project. This forms a DDE link with the VISion console.
' Calling GPIBserver.Boot will try to establish the server
' ------------------------------------------------------------------------------------

  Private Sub vi5_debug(txt$)
          If vi5_DDE_Server_Running = 1 Then
             Else
                 Debug.Print txt$
                 
          End If
          If vi5_spyrequest = True Then
             vi5_spyback = vi5_spyback + txt$ + vbCrLf
          End If
  End Sub

' -----------------------------------------------------------------
' GPIBreset
'
' resets GPIB board. and claims board as primary controller
' sets runlevel to 1 upon success.
' sets runlevel to -1 upon failure.
'
' Function needs to be called to start GPIB stack.
'
' -----------------------------------------------------------------

  Public Sub GPIBinit()
    vi5_debug "' GPIBCore " + gpibversion$ + " " + gpibbuild$
    vi5_debug "'   (c) 1992 - 2000 V.Himpe All Rights Reserved"
    vi5_debug "'   (c) 1996 - 2000 Alcatel Microelectronics"
    vi5_debug "'   Released under OpenSource policy"
    vi5_debug "' "
    vi5_debug "'   Run initiated at " + Date$ + "/" + Time$
    vi5_debug "'-------------------------------------------------------"
    vi5_debug "GPIBinit    "
    Dim vi5_reset_failed, x
    vi5_reset_failed = 0
    vi5_runlevel = 0  ' startup runlevel
    
    Dim status%
    
    status% = ilsic(0)
    If (ibsta And EERR) Then
       vi5_debug "' GPIBCORE : ERROR #12 : Could not become Controller in Charge "
       vi5_reset_failed = 1
    End If
    status% = ilonl(0, 0)
    If (ibsta And EERR) Then
       vi5_debug "' GPIBCORE : ERROR #10 : Could not take board OFF-line "
       vi5_reset_failed = 1
    End If
    status% = ilonl(0, 1)
    If (ibsta And EERR) Then
       vi5_debug "' GPIBCORE : ERROR #11 : Could not take board ON-line "
       vi5_reset_failed = 1
    End If
    status% = ilsic(0)
    If (ibsta And EERR) Then
       vi5_debug "' GPIBCORE : ERROR #12 : Could not become Controller in Charge "
       vi5_reset_failed = 1
    End If
    
    For x = 0 To 31  ' reset all descriptors
        vi5_instrument(x).runstate = False
        vi5_instrument(x).handle = 0
        vi5_instrument(x).descriptor = ""
    Next x
    
    status% = iltmo(0, T3s) ' set 3 seconds as standard timeout period
    status% = ileot(0, 1)  ' send EOI with last byte
    
    If (status% And EERR) Then
       If iberr = ENEB Then
         vi5_debug "' GPIBCORE : ERROR -1 : No GPIB board found"
         vi5_reset_failed = 1
       End If
       If iberr = EDVR Then
         vi5_debug "' GPIBCORE : ERROR -2 : No GPIB driver present "
        vi5_reset_failed = 1
       End If
    End If
    
    ' patch 4:
    ' patch9 ibconfig 0, 7, 1 ' set automatic serial polling
    
    ilsre 0, 1 ' make devices go to remote upon first access
    
    If vi5_reset_failed = 1 Then
       vi5_debug "' GPIBCORE : ERROR #999 : BUS Reset *FAILED* : GPIB DISABLED "
       vi5_runlevel = -1 ' bus arbitration failed : Core halted
    Else
       vi5_debug "' GPIBCORE : RESET entire GPIB STACK"
       vi5_debug "'          > All devices handles have been released"
       vi5_debug "'          > GPIBcore is *IN CONTROL*"
       vi5_runlevel = 1  ' bus arbitering succeeded. Computer is in control
    End If
    NextStep
End Sub

' -----------------------------------------------------------------
' -----------------------------------------------------------------

Public Function GPIBopen(address, Optional comment)
           Dim vi5_tmp As Integer
           Dim description$
           
           Dim listen%, status%
           vi5_inputbuffer = ""
           
           If vi5_runlevel = 1 Then
              If IsMissing(comment) Then
                 description$ = ""
                 vi5_debug "GPIBopen " & address
              Else
                 description$ = comment
                 vi5_debug "GPIBopen " & address & "," & comment
              End If
               If vi5_instrument(address).runstate = False Then
                 status% = illn(0, address, 0, listen%)
                 If listen% <> 0 Then
                    vi5_instrument(address).handle = ildev(0, address, 0, T3s, 1, 0)
                    If ibsta And EERR Then ' failed to create handle
                       vi5_debug "' GPIBCORE : ERROR #3 : Failure to open channel to device"
                      Else ' handle creation succeeded.
                       ' patch9 four lines below
                       'ilclr vi5_instrument(Address).handle ' clear device
                       'If (ibsta And EERR) Then
                       '   vi5_debug "' GPIBCORE : ERROR #4 : Unable to clear device channel "
                       'Else
                          
                          vi5_instrument(address).runstate = True
                          ilonl vi5_instrument(address).handle, 1  ' might be buggy sometimes
                          status% = ileot(address, 1)  ' send EOI with last byte patch 07
                          ilwrt vi5_instrument(address).handle, "*CLS", 4 ' clear the instrument
                          
                          vi5_instrument(address).descriptor = Trim$(description$)
                          If vi5_instrument(address).descriptor = "" Then ' if no descriptor given try to find it
                             
                             ilwrt vi5_instrument(address).handle, "*IDN?", 5
                             ilrd vi5_instrument(address).handle, vi5_inputbuffer, 65300
                             vi5_tmpbuffer = Trim$(Left$(vi5_inputbuffer, 100)) + ">>,NON SCPI device,dummy"
                             vi5_tmp = InStr(vi5_tmpbuffer, ",") + 1
                             vi5_tmpbuffer = Left$(vi5_tmpbuffer, InStr(vi5_tmp, vi5_tmpbuffer, ",") - 1)
                             'If InStr(vi5_tmpbuffer, ">>,NON SCPI") > 0 Then
                             '  vi5_tmpbuffer = "UNKNOWN, Non SCPI device"
                               ' make non scpi ID routine here
                               
                             'End If
                             
                             If Left$(vi5_tmpbuffer, 11) = ">>,NON SCPI" Then
                                vi5_tmpbuffer = "UNKNOWN, Non SCPI device"
                             End If
                                vi5_instrument(address).descriptor = Trim$(vi5_tmpbuffer)
                             GPIBopen = Trim$(Left$(vi5_inputbuffer, 300)) 'vi5_instrument(Address).descriptor
                          End If
                          
                          vi5_debug "' GPIBCORE : Successfully obtained link to address " & address
                          vi5_debug "'          > Runstate   :" & vi5_instrument(address).runstate
                          vi5_debug "'          > Handle     :" & vi5_instrument(address).handle
                          vi5_debug "'          > Descriptor :" & Trim$(vi5_instrument(address).descriptor)
                          
                       End If
                 Else
                     vi5_debug "' GPIBCORE : Warning #2 No device at address " & address
                 End If
              Else
                 vi5_debug "' GPIBCORE : ERROR #2 : Attempted to open an already open device"
                 VI5_raise_error 2, "Attempted to reopen instrument descriptor for address " & address
              End If
            Else
               vi5_debug "' GPIBCORE : ERROR #1 : Not in RUNLEVEL 1 :> BUS ACCESS DENIED "
           End If
           NextStep
End Function

' *******************************************************************
' Closes and instrument ont he bus.Instrument is returned to it's
' LOCAL state
' *******************************************************************

Public Sub GPIBclose(address)
       Dim status%
       If vi5_runlevel = 1 Then
       vi5_debug "GPIBclose " & address
       If vi5_instrument(address).runstate = True Then
          vi5_instrument(address).runstate = False
          status% = illoc(vi5_instrument(address).handle)
          status% = ilonl(vi5_instrument(address).handle, 0)
          If status% And iberr Then
             vi5_debug "' GPIBCORE : ERROR : There are error(s) ! GPIBclose "
          Else
              vi5_instrument(address).runstate = False
          End If
       Else
          vi5_debug "' GPIBCORE : Warning #1 : attempted to release unassigned handle "
       End If
       Else
          vi5_debug "' GPIBCORE : ERROR #1 : Not in RUNLEVEL 1 :> BUS ACCESS DENIED "
       End If
       NextStep
End Sub

' -------------------------------------------------
' VI5_Write
'
' sends a string to the device at the specified address.
' checks runmap to verify device exists and has a valid handle
' ------------------------------------------------------------

Public Sub GPIBwrite(address, cmd$)
       Dim status%
       If vi5_runlevel = 1 Then
         If vi5_instrument(address).runstate = True Then
         
          status% = ilwrt(vi5_instrument(address).handle, cmd$, Len(cmd$))
          If status% And EERR Then
            
            vi5_debug "' GPIBCORE : GPIBWRITE failed for address " & address
            If iberr = EABO Then
                 vi5_debug "         > timeout during transaction"
            End If
            
          Else
            vi5_debug "GPIBWRITE " & address & "," & Chr$(34) & cmd$ & Chr$(34)
          End If
          Else
            vi5_debug "' DEFER : GPIBWRITE " & address & "," & Chr$(34) & cmd$ & Chr$(34)
          End If
       Else
          vi5_debug "' GPIBCORE : ERROR #1 : Not in RUNLEVEL 1 :> BUS ACCESS DENIED "
       End If
       NextStep
End Sub
' -------------------------------------------------
' VI5_Write_from_file
'
' sends a string to the device at the specified address.
' checks runmap to verify device exists and has a valid handle
' ------------------------------------------------------------

Public Sub VI5_write_from_file(address, file$)
       Dim status%
       If vi5_runlevel = 1 Then
         If vi5_instrument(address).runstate = True Then
          status% = ilwrtf(vi5_instrument(address).handle, file$)
          If status% And EERR Then
             vi5_debug "' GPIBCORE : GPIBWRITEfromFILE failed for address " & address
             
             If iberr = EABO Then
                vi5_debug "'            > ERROR 14 : Timout during transaction "
             End If
             If iberr = EFSO Then
                vi5_debug "'            > ERROR 15 : File " & file$ & " could not be accessed "
             End If
             
          Else
             vi5_debug "GPIBWRITEfromFILE " & address & "," & Chr$(34) & file$ & Chr$(34)
          End If
          Else
          vi5_debug "' DEFER : GPIBWRITEfromFILE " & address & "," & Chr$(34) & file$ & Chr$(34)
         End If
       Else
          vi5_debug "' GPIBCORE : ERROR #1 : Not in RUNLEVEL 1 :> BUS ACCESS DENIED "
       End If
End Sub


' -----------------------------------------------------
' nice to have but not critical stuff
' -----------------------------------------------------

Public Function GPIBfind(address)
     Dim status%, listen%, x%
     
     If vi5_runlevel = 1 Then
        x% = address
        status% = illn(0, x%, 0, listen%)
        If listen% <> 0 Then
           vi5_debug "' GPIBCORE : Device found (" & address & ")"
           GPIBfind = True
        End If
      Else
        vi5_debug "' GPIBCORE : ERROR #1 : Not in RUNLEVEL 1 :> BUS ACCESS DENIED "
      End If
End Function



' **********************************************************
'
' **********************************************************

   Private Sub VI5_raise_error(errorstyle, txt$)
        ' vi5_debug "ERROR " & errorstyle & ":" & txt$
        Select Case errorstyle
        Case 0
        Case Else
             MsgBox "ERROR " & errorstyle & ":" & txt$
        End Select
    End Sub



' -------------------------------------------------------------------
' Linking of the VI5 functions to the GPIBcore functions
' -------------------------------------------------------------------



' parse a gpib command literally on the board
  Public Sub GPIBparse(commando$)
         MsgBox " GPIBPARSE is no longer implemented. Change your program"
         End
  End Sub

  
' *******************************************************************
' Opens and instrument on the bus. The instrument goes in REMOTE
' *******************************************************************
 
' *******************************************************************
' Resets the GPIB Card . Use for emergency situations.
' *******************************************************************
  Public Sub GPIBreset()
         vi5_debug "GPIBreset"
         vi5_debug "     > Redirected to GPIBinit handler"
         GPIBinit
  End Sub



' *******************************************************************
' Writes a file containing commands to an instrument
' *******************************************************************
  Public Sub GPIBwriteFILE(address, file$)
    VI5_write_from_file address, file$
    NextStep
  End Sub

' *******************************************************************
' Defers the calls to the instruments
' *******************************************************************
  Public Sub GPIBdefer(state As Boolean, Optional address)
    If IsMissing(address) Then
       vi5_defer = state
    Else
       vi5_instrument(address).deferred = state
    End If
  End Sub

' *******************************************************************
' Closes the Entire system
' *******************************************************************
  Public Sub Bye()
      vi5_debug "Bye"
      vi5_debug "'  > Redirected to GPIBinit"
      GPIBinit
  End Sub
  
' *******************************************************************
' GPIBtimeout
' *******************************************************************
  Public Sub GPIBtimeout(seconds, Optional device)
         Dim x
         If vi5_runlevel = 1 Then
         Select Case seconds
         Case 0 To 1  ' 1 sec
           x = T1s
         Case 2 To 3  ' 3 sec
           x = T3s
         Case 4 To 10
           x = T10s
         Case 11 To 30
           x = T30s
         Case 31 To 100
           x = T100s
         Case 101 To 299
           x = T300s
         Case 301 To 1000 ' 1000 sec
           x = T1000s
         Case Else   ' 10 sec
           x = T10s
         End Select
         
          If IsMissing(device) Then
             iltmo 0, x
             vi5_debug "GPIBtimeout " & seconds
             vi5_debug "' Card timeout set to " & seconds & "seconds"
          Else
             iltmo vi5_instrument(device).handle, x
             vi5_debug "GPIBtimeout " & seconds & "," & device
             vi5_debug "' Device timeout for " & device & " set to : " & seconds
          End If
         Else
          vi5_debug "GPIBCORE : ERROR #1 : Not in RUNLEVEL 1 :> BUS ACCESS DENIED "
       End If
  End Sub

' enable stracing of an instrument
  Public Sub GPIBtron(address)
      vi5_instrument(address).tracing = True
  End Sub

' disables tracing of an instrument
  Public Sub GPIBtroff(address)
      vi5_instrument(address).tracing = False
  End Sub



' *******************************************************************
' Reads data from an instrument
' *******************************************************************
Public Sub GPIBread(address, cmd$)
       Dim status%
       Dim readsuccess
       readsuccess = 0
       
       If vi5_runlevel = 1 Then
          If vi5_instrument(address).runstate = True Then
             vi5_debug "GPIBREAD " & address & "," & Chr$(34) & cmd$ & Chr$(34)
             If Len(Trim$(cmd$)) > 0 Then
                status% = ilwrt(vi5_instrument(address).handle, cmd$, Len(cmd$))
                If status% And EERR Then
                   vi5_debug "' GPIBCORE : GPIBWRITE failed for address " & address
                   If iberr = EABO Then
                      vi5_debug "'          > timeout during transaction"
                   End If
                Else
                   
                End If
             End If
             status% = ilrd(vi5_instrument(address).handle, vi5_inputbuffer, 65300)
             If status% And EERR Then
                   vi5_debug "' GPIBCORE : GPIBWREAD failed for address " & address
                   If iberr = EABO Then
                      vi5_debug "'          > timeout during transaction"
                   End If
                Else
            
                   vi5_tmpbuffer = Left$(vi5_inputbuffer, ibcntl)
                   If Len(vi5_tmpbuffer) > 30 Then
                   vi5_debug "' RETURN >" & Left$(vi5_tmpbuffer, 25) & "..."
                   Else
                   vi5_debug "' RETURN >" & vi5_tmpbuffer
                   End If
                   ib$ = vi5_tmpbuffer
                   ibret = Val(vi5_tmpbuffer)
                End If
          Else
            vi5_debug "DEFER : GPIBREAD " & address & "," & Chr$(34) & cmd$ & Chr$(34)
          End If
       Else
          vi5_debug "GPIBCORE : ERROR #1 : Not in RUNLEVEL 1 :> BUS ACCESS DENIED "
       End If
      
End Sub

Sub GPIBspoll(address)
       If vi5_runlevel = 1 Then
          If vi5_instrument(address).runstate = True Then
             vi5_debug "GPIBspoll " & address
             
          Else
            vi5_debug "DEFER : GPIBspoll " & address
          End If
       Else
          vi5_debug "GPIBCORE : ERROR #1 : Not in RUNLEVEL 1 :> BUS ACCESS DENIED "
       End If
    
End Sub

' **********************************************************
' Thunked write Access
' **********************************************************
  Public Sub GPIBThunkWrite(address, dta$, Optional count)
      MsgBox "GPIBThunkwrite : thunking no longer possible . Change your program '"
      End
  End Sub
' **********************************************************
' Thunked read access
' **********************************************************
  Public Function GPIBThunkRead(address, count)
      MsgBox "GPIBThunkread : thunking no longer possible . Change your program '"
      End
  End Function

  
  
' singlestep GPIB operations !

Public Sub GPIBsinglestep(state As Boolean)
    vi5_singlestep = state
End Sub

' *******************************************************************
' Enters text into the log window on the Vision ControlCenter
' *******************************************************************
   Public Sub logentry(entrytxt$)
          vi5_debug ":: " & entrytxt$
   End Sub


Private Sub NextStep()
       Dim answer As Integer
       If vi5_singlestep = True Then
          answer = MsgBox("Step executed" + Chr$(13) + Chr$(10) + " Click Yes to continue stepping" + Chr$(13) + Chr$(10) + "Click No to stop stepping", vbInformation + vbYesNo, " VIsion Debugger message")
          If answer = vbNo Then
             vi5_singlestep = False
          End If
       End If
End Sub


' **********************************************************
' Miscellanious support functions
' **********************************************************

' generate a delay in seconds
Public Sub delay(seconds As Integer)
       Dim cca As Integer
       For cca = 1 To seconds
           Sleep 998
       Next cca
End Sub

' generate a millisecond delay
Public Sub MicroDelay(milliseconds As Integer)
       Dim cca As Integer
       Sleep milliseconds
End Sub


Public Function Bin$(value)  ' returns binary equivalent up to 65535
       Dim b As Integer
       Dim result$
       Dim TMPvalue
       TMPvalue = value
       If Abs(TMPvalue) <> TMPvalue Then  ' negative number
          TMPvalue = 65536 + TMPvalue    'convert to 2 complement
             ' we use a trick here
             ' instead of inverting all bits and adding one
             ' we take the largest number available , add one
             ' and then substract the original value
       End If
       For b = 15 To 0 Step -1
           If (TMPvalue And (2 ^ b)) = (2 ^ b) Then
              result$ = result$ + "1"
             Else
              result$ = result$ + "0"
           End If
       Next b
       Bin$ = result$
End Function

' **********************************************************
' SSTR$  Stripped STR function
' **********************************************************
  Public Function sStr$(value)
         sStr$ = Trim$(Str$(value))
  End Function



' -*-*-*-*-*-*--*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
' work in progress . Does not do Twos complement yet
' -*-*-*-*-*-*--*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

Public Function vval(strng$)
    Dim tmp, x As Integer
    Dim twocmpl As Boolean
    tmp = 0
    twocmpl = False
    strng$ = UCase$(Trim$(strng$))
    If Left$(strng$, 2) = "&B" Then  ' if binary format
       strng$ = Right$(strng$, Len(strng$) - 2)
       
       ' check for byte or word boundary and patch string
       
       Select Case Len(strng$)
           Case 1 To 7
              strng$ = String$(8 - Len(strng$), "0") + strng$
           Case 9 To 15
              strng$ = String$(16 - Len(strng$), "0") + strng$
           Case 17 To 31
              strng$ = String$(32 - Len(strng$), "0") + strng$
       End Select
       
     '  Debug.Print strng$
       
       ' scan string
       
       For x = 1 To Len(strng$)
          If x = 1 Then  ' if MSB true then flip into twos complement mode
             If Left$(strng$, 1) = "1" Then
                twocmpl = True
             Else
                tmp = tmp * 2
                If Mid$(strng$, x, 1) = "1" Then
                    tmp = tmp + 1
                End If
             End If
            Else
             tmp = tmp * 2
             If Mid$(strng$, x, 1) = "1" Then
                tmp = tmp + 1
             End If
          End If
       Next x
       If twocmpl = True Then
          tmp = -tmp
       End If
       vval = tmp
    Else
    vval = Val(strng$)
    End If
End Function


' ------------------------------------------------
' DDE link for optional debugger
' ------------------------------------------------

Public Function GPIBspy(cmd$)
    Dim x$, Y$
    Dim addr, para
    cmd$ = UCase$(cmd$)
    cmd$ = Trim$(cmd$) + " "
    x$ = Trim$(Left$(cmd$, InStr(cmd$, " ")))
    Y$ = Trim$(Right$(cmd$, Len(cmd$) - Len(x)))
    Select Case x$
       Case "GPIBINIT"
          vi5_spyrequest = True
          GPIBinit
          vi5_spyrequest = False
          GPIBspy = vi5_spyback
          vi5_spyback = ""
       Case "GPIBOPEN"
          vi5_spyrequest = True
          GPIBopen Val(Y$)
          vi5_spyrequest = False
          GPIBspy = vi5_spyback
          vi5_spyback = ""
       Case "GPIBCLOSE"
          vi5_spyrequest = True
          GPIBclose Val(Y$)
          vi5_spyrequest = False
          GPIBspy = vi5_spyback
          vi5_spyback = ""
       Case "HELLO"
       GPIBspy = "Hello there. You triggered the GPIBspy"
       Case "WHODUNIT"
       GPIBspy = "The one and only : Ultravinnie "
       Case "GPIBWRITE"
          addr = Val(Y$)
          Y$ = Right$(Y$, Len(Y$) - 2)
          vi5_spyrequest = True
          GPIBwrite addr, Y$
          vi5_spyrequest = False
          GPIBspy = vi5_spyback
          vi5_spyback = ""
       Case ""
          GPIBspy = ">"
       Case "SNOOP"
          addr = Val(Left$(Y$, InStr(Y$, " ")))
          para = Val(Right$(Y$, Len(Y$) - InStr(Y$, " ")))
          GPIBspy = snoop(addr, para)
       Case Else
          GPIBspy = "syntax error >"
    End Select
End Function

Private Function snoop(address, parameter) As String
    Select Case address
    Case -1
    
    Case 0 To 32
       Select Case parameter
          Case 1
             snoop = Str$(vi5_instrument(address).handle)
          Case 2
             snoop = vi5_instrument(address).descriptor
          Case 3
             snoop = vi5_instrument(address).truename
          Case 4
             snoop = vi5_instrument(address).runstate
          Case 5
             snoop = vi5_instrument(address).tracing
          Case 6
             snoop = vi5_instrument(address).deferred
          Case Else
             snoop = "-VOID-"
       End Select
    Case Else
       snoop = "' Syntax Error"
    End Select
End Function




' ------------------------------------------------------------------------------------------
' Glue code for National Instrument GPIB-32.DLL
' The routines below form the interface between Visual Basic and the NI dll
' ------------------------------------------------------------------------------------------


Private Function ConvertLongToInt(LongNumb As Long) As Integer
  If (LongNumb And &H8000&) = 0 Then
      ConvertLongToInt = LongNumb And &HFFFF&
  Else
    ConvertLongToInt = &H8000 Or (LongNumb And &H7FFF&)
  End If
End Function

Private Sub RegisterGPIBGlobals()
    Dim rc As Long
    rc = RegisterGpibGlobalsForThread(Longibsta, Longiberr, Longibcnt, ibcntl)
    If (rc = 0) Then
      GPIBglobalsRegistered = 1
    ElseIf (rc = 1) Then
      rc = UnregisterGpibGlobalsForThread
      rc = RegisterGpibGlobalsForThread(Longibsta, Longiberr, Longibcnt, ibcntl)
      GPIBglobalsRegistered = 1
    ElseIf (rc = 2) Then
      rc = UnregisterGpibGlobalsForThread
      ibsta = &H8000
      iberr = EDVR
      ibcntl = &HDEAD37F0
    ElseIf (rc = 3) Then
      rc = UnregisterGpibGlobalsForThread
      ibsta = &H8000
      iberr = EDVR
      ibcntl = &HDEAD37F0
    Else
      ibsta = &H8000
      iberr = EDVR
      ibcntl = &HDEAD37F0
    End If
End Sub

Private Sub UnregisterGPIBGlobals()
    Dim rc As Long
    rc = UnregisterGpibGlobalsForThread
    GPIBglobalsRegistered = 0
End Sub

Private Sub copy_ibvars()
    ibsta = ConvertLongToInt(Longibsta)
    iberr = CInt(Longiberr)
    ibcnt = ConvertLongToInt(ibcntl)
End Sub


Private Function ilclr(ByVal ud As Integer) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ilclr = ConvertLongToInt(ibclr32(ud))
    Call copy_ibvars
End Function

Private Function ildev(ByVal bdid As Integer, ByVal pad As Integer, ByVal sad As Integer, ByVal tmo As Integer, ByVal eot As Integer, ByVal eos As Integer) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ildev = ConvertLongToInt(ibdev32(bdid, pad, sad, tmo, eot, eos))
    Call copy_ibvars
End Function

Private Function ileos(ByVal ud As Integer, ByVal v As Integer) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ileos = ConvertLongToInt(ibeos32(ud, v))
    Call copy_ibvars
End Function

Private Function ileot(ByVal ud As Integer, ByVal v As Integer) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ileot = ConvertLongToInt(ibeot32(ud, v))
    Call copy_ibvars
End Function

Private Function illn(ByVal ud As Integer, ByVal pad As Integer, ByVal sad As Integer, ln As Integer) As Integer
    Dim tmpln As Long
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    illn = ConvertLongToInt(ibln32(ud, pad, sad, tmpln))
    ln = ConvertLongToInt(tmpln)
    Call copy_ibvars
End Function

Private Function illoc(ByVal ud As Integer) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    illoc = ConvertLongToInt(ibloc32(ud))
    Call copy_ibvars
End Function

Private Function ilonl(ByVal ud As Integer, ByVal v As Integer) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ilonl = ConvertLongToInt(ibonl32(ud, v))
    Call copy_ibvars
End Function

Private Function ilrd(ByVal ud As Integer, buf As String, ByVal cnt As Long) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ilrd = ConvertLongToInt(ibrd32(ud, ByVal buf, cnt))
    Call copy_ibvars
End Function

Private Function ilrdf(ByVal ud As Integer, ByVal filename As String) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ilrdf = ConvertLongToInt(ibrdf32(ud, ByVal filename))
    Call copy_ibvars
End Function

Private Function ilsic(ByVal ud As Integer) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ilsic = ConvertLongToInt(ibsic32(ud))
    Call copy_ibvars
End Function

Private Function ilsre(ByVal ud As Integer, ByVal v As Integer) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ilsre = ConvertLongToInt(ibsre32(ud, v))
    Call copy_ibvars
End Function

Private Function iltmo(ByVal ud As Integer, ByVal v As Integer) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    iltmo = ConvertLongToInt(ibtmo32(ud, v))
    Call copy_ibvars
End Function

Private Function iltrg(ByVal ud As Integer) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    iltrg = ConvertLongToInt(ibtrg32(ud))
    Call copy_ibvars
End Function

Private Function ilwrt(ByVal ud As Integer, ByVal buf As String, ByVal cnt As Long) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ilwrt = ConvertLongToInt(ibwrt32(ud, ByVal buf, cnt))
    Call copy_ibvars
End Function

Private Function ilwrtf(ByVal ud As Integer, ByVal filename As String) As Integer
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If
    ilwrtf = ConvertLongToInt(ibwrtf32(ud, ByVal filename))
    Call copy_ibvars
End Function

Sub ibconfig(ByVal bdid As Integer, ByVal opt As Integer, ByVal v As Integer)
' Check to see if GPIB Global variables are registered
    If (GPIBglobalsRegistered = 0) Then
      Call RegisterGPIBGlobals
    End If

' Call the 32-bit DLL.
    Call ibconfig32(bdid, opt, v)

    Call copy_ibvars
End Sub







